package org.eclipse.albireo.examples.plugin.views;

import java.awt.Color;
import java.awt.Component;
import java.awt.GridLayout;
import java.awt.MenuItem;
import java.awt.Point;
import java.awt.PopupMenu;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.SwingConstants;

import org.eclipse.albireo.core.AwtEnvironment;
import org.eclipse.albireo.core.SwingControl;
import org.eclipse.albireo.core.SwtPopupRegistry;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.part.ViewPart;

public class AwtPopupView extends ViewPart {

    private SwingControl swingControl;

    public AwtPopupView() {
        // TODO Auto-generated constructor stub
    }

    public void createPartControl(Composite parent) {
        swingControl = new SwingControl(parent, SWT.NONE) {

            protected JComponent createSwingComponent() {
                JPanel panel = new JPanel(new GridLayout(3, 1, 20, 20));
                
                JTextField swingLabel = new JTextField("Open Swing popup here", SwingConstants.CENTER);
                swingLabel.setFocusable(true);
                swingLabel.setBorder(BorderFactory.createLineBorder(Color.GREEN));
                addSwingPopup(swingLabel);
                panel.add(swingLabel);
                
                JTextField awtLabel = new JTextField("Open AWT popup here", SwingConstants.CENTER);
                awtLabel.setFocusable(true);
                awtLabel.setBorder(BorderFactory.createLineBorder(Color.BLUE));
                addAwtPopup(awtLabel);
                panel.add(awtLabel);
                
                JTextField swtPopupLabel = new JTextField("Open SWT popup here", SwingConstants.CENTER);
                //swtPopupLabel.setFocusable(true);
                swtPopupLabel.setBorder(BorderFactory.createLineBorder(Color.RED));
                addSwtPopup(swtPopupLabel);
                panel.add(swtPopupLabel);
                
                return panel;

            }

            public Composite getLayoutAncestor() {
                return getParent();
            }

        };

    }
    
    private void addSwtPopup(final Component c) {
        swingControl.getDisplay().asyncExec(new Runnable() {
            public void run() {
                Shell parent = AwtEnvironment.getInstance(swingControl.getDisplay()).getSwtPopupParent(swingControl);
                Menu menu = new Menu(parent, SWT.POP_UP);
                final org.eclipse.swt.widgets.MenuItem item = new org.eclipse.swt.widgets.MenuItem(
                        menu, SWT.PUSH);
                item.setText("SWT Item 1");
                item.addSelectionListener(new SelectionListener() {
                    public void widgetDefaultSelected(SelectionEvent e) {
                        showMessage(item);
                    }
                    public void widgetSelected(SelectionEvent e) {
                        showMessage(item);
                    }
                });
                final org.eclipse.swt.widgets.MenuItem item2 = new org.eclipse.swt.widgets.MenuItem(menu, SWT.PUSH);
                item2.setText("SWT Item 2");
                item2.addSelectionListener(new SelectionListener() {
                    public void widgetDefaultSelected(SelectionEvent e) {
                        showMessage(item2);
                    }
                    public void widgetSelected(SelectionEvent e) {
                        showMessage(item2);
                    }
                });
                SwtPopupRegistry.getInstance().setMenu(c, false, menu);
            }

            private void showMessage(final org.eclipse.swt.widgets.MenuItem item2) {
                MessageDialog.openConfirm(swingControl.getShell(), item2.getText(), item2.getText());
            }
        });
    }

    private void addAwtPopup(final JComponent c) {
        ActionListener listener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JOptionPane.showMessageDialog(swingControl.getAWTHierarchyRoot(), "Item selected");
            }
        };
        final PopupMenu popup = new PopupMenu();
        MenuItem item = new MenuItem("AWT Item 1");
        item.addActionListener(listener);
        popup.add(item);

        item = new MenuItem("AWT Item 2");
        item.addActionListener(listener);
        popup.add(item);

        c.add(popup);
        c.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) { 
                if (e.getKeyCode() == KeyEvent.VK_F10 && 
                        ((e.getModifiers() & KeyEvent.SHIFT_MASK) != 0)) {
                    Rectangle r = c.getVisibleRect();
                    Point point = new Point(r.x + r.width/2,
                                            r.y + r.height/2);
                    popup.show(c, point.x, point.y);
                }
            }
            
        });
        c.addMouseListener(new MouseAdapter() {

            public void mousePressed(MouseEvent e) {
                show(popup, e);
            }

            public void mouseReleased(MouseEvent e) {
                show(popup, e);
            }

            private void show(final PopupMenu popup, MouseEvent e) {
                if (e.isPopupTrigger()) {
                    popup.show(e.getComponent(), e.getX(), e.getY());
                }
            }
            
        });
    }

    private void addSwingPopup(final JComponent c) {
        ActionListener listener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                JOptionPane.showMessageDialog(swingControl.getAWTHierarchyRoot(), "Item selected");
            }
        };
        final JPopupMenu popup = new JPopupMenu();
        JMenuItem item = new JMenuItem("Swing Item 1");
        item.addActionListener(listener);
        popup.add(item);
        item = new JMenuItem("Swing Item 2");
        item.addActionListener(listener);
        popup.add(item);
        
        // We still run on Java 1.4, so can't use the new JComponent.setComponentPopupMenu
        // method
        c.addKeyListener(new KeyAdapter() {
            public void keyPressed(KeyEvent e) { 
                if (e.getKeyCode() == KeyEvent.VK_F10 && 
                        ((e.getModifiers() & KeyEvent.SHIFT_MASK) != 0)) {
                    Rectangle r = c.getVisibleRect();
                    Point point = new Point(r.x + r.width/2,
                                            r.y + r.height/2);
                    popup.show(c, point.x, point.y);
                }
            }
            
        });
        c.addMouseListener(new MouseAdapter() {

            public void mousePressed(MouseEvent e) {
                show(popup, e);
            }

            public void mouseReleased(MouseEvent e) {
                show(popup, e);
            }

            private void show(final JPopupMenu popup, MouseEvent e) {
                if (e.isPopupTrigger()) {
                    popup.show(e.getComponent(), e.getX(), e.getY());
                }
            }
            
        });
        
    }
    public void setFocus() {
        swingControl.setFocus();
    }
}
