package org.eclipse.albireo.examples.plugin.views;

import java.awt.Component;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.HierarchyBoundsListener;
import java.awt.event.HierarchyEvent;
import java.awt.event.HierarchyListener;
import java.awt.event.InputMethodEvent;
import java.awt.event.InputMethodListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.EventObject;

import javax.swing.JComponent;
import javax.swing.JPanel;

import org.eclipse.albireo.core.SwingControl;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.ViewPart;

/**
 * This view shows nothing; it logs events.
 * For debugging purposes.
 */
public class EventLoggerView extends ViewPart {

    public static String ID = "org.eclipse.albireo.examples.plugin.eventLogger"; //$NON-NLS-1$
    private SwingControl swingControl;

    public void createPartControl(final Composite parent) {
        swingControl = new SwingControl(parent, SWT.NONE) {
            protected JComponent createSwingComponent() {
                JPanel panel = new JPanel();
                return panel;
            }
            protected void afterComponentCreatedAWTThread() {
                for (Component c = getSwingComponent(); c != null; c = c.getParent()) {
                    c.addComponentListener(new LoggingComponentListener(c));
                    c.addHierarchyListener(new LoggingHierarchyListener(c));
                    c.addHierarchyBoundsListener(new LoggingHierarchyBoundsListener(c));
                    c.addMouseListener(new LoggingMouseListener(c));
                    c.addMouseMotionListener(new LoggingMouseMotionListener(c));
                    c.addMouseWheelListener(new LoggingMouseWheelListener(c));
                    c.addKeyListener(new LoggingKeyListener(c));
                    c.addFocusListener(new LoggingFocusListener(c));
                    c.addInputMethodListener(new LoggingInputMethodListener(c));
                    c.addPropertyChangeListener(new LoggingPropertyChangeListener(c));
                }
            }

            public Composite getLayoutAncestor() {
                return parent;
            }
        };
        
        
    }

    static void logEvent(Component c, EventObject event) {
        System.err.println("@"+System.currentTimeMillis()
                           + " " + c.getClass().getName() + "@" + System.identityHashCode(c)
                           + " " + event);
    }

    static class LoggingComponentListener implements ComponentListener {
        private Component component;
        LoggingComponentListener(Component c) {
            component = c;
        }
        public void componentResized(ComponentEvent event) {
            logEvent(component, event);
        }
        public void componentMoved(ComponentEvent event) {
            logEvent(component, event);
        }
        public void componentShown(ComponentEvent event) {
            logEvent(component, event);
        }
        public void componentHidden(ComponentEvent event) {
            logEvent(component, event);
        }
    }

    static class LoggingHierarchyListener implements HierarchyListener {
        private Component component;
        LoggingHierarchyListener(Component c) {
            component = c;
        }
        public void hierarchyChanged(HierarchyEvent event) {
            logEvent(component, event);
        }
    }

    static class LoggingHierarchyBoundsListener implements HierarchyBoundsListener {
        private Component component;
        LoggingHierarchyBoundsListener(Component c) {
            component = c;
        }
        public void ancestorMoved(HierarchyEvent event) {
            logEvent(component, event);
        }
        public void ancestorResized(HierarchyEvent event) {
            logEvent(component, event);
        }
    }

    static class LoggingMouseListener implements MouseListener {
        private Component component;
        LoggingMouseListener(Component c) {
            component = c;
        }
        public void mouseClicked(MouseEvent event) {
            logEvent(component, event);
        }
        public void mousePressed(MouseEvent event) {
            logEvent(component, event);
        }
        public void mouseReleased(MouseEvent event) {
            logEvent(component, event);
        }
        public void mouseEntered(MouseEvent event) {
            logEvent(component, event);
        }
        public void mouseExited(MouseEvent event) {
            logEvent(component, event);
        }
    }

    static class LoggingMouseMotionListener implements MouseMotionListener {
        private Component component;
        LoggingMouseMotionListener(Component c) {
            component = c;
        }
        public void mouseDragged(MouseEvent event) {
            logEvent(component, event);
        }
        public void mouseMoved(MouseEvent event) {
            logEvent(component, event);
        }
    }

    static class LoggingMouseWheelListener implements MouseWheelListener {
        private Component component;
        LoggingMouseWheelListener(Component c) {
            component = c;
        }
        public void mouseWheelMoved(MouseWheelEvent event) {
            logEvent(component, event);
        }
    }

    static class LoggingKeyListener implements KeyListener {
        private Component component;
        LoggingKeyListener(Component c) {
            component = c;
        }
        public void keyTyped(KeyEvent event) {
            logEvent(component, event);
        }
        public void keyPressed(KeyEvent event) {
            logEvent(component, event);
        }
        public void keyReleased(KeyEvent event) {
            logEvent(component, event);
        }
    }

    static class LoggingFocusListener implements FocusListener {
        private Component component;
        LoggingFocusListener(Component c) {
            component = c;
        }
        public void focusGained(FocusEvent event) {
            logEvent(component, event);
        }
        public void focusLost(FocusEvent event) {
            logEvent(component, event);
        }
    }

    static class LoggingInputMethodListener implements InputMethodListener {
        private Component component;
        LoggingInputMethodListener(Component c) {
            component = c;
        }
        public void inputMethodTextChanged(InputMethodEvent event) {
            logEvent(component, event);
        }
        public void caretPositionChanged(InputMethodEvent event) {
            logEvent(component, event);
        }
    }

    static class LoggingPropertyChangeListener implements PropertyChangeListener {
        private Component component;
        LoggingPropertyChangeListener(Component c) {
            component = c;
        }
        public void propertyChange(PropertyChangeEvent event) {
            logEvent(component, event);
        }
        
    }

    public void setFocus() {
        swingControl.setFocus();
    }
}
