package org.eclipse.albireo.examples.plugin.views;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;

import org.eclipse.albireo.core.SwingControl;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.ViewPart;

/**
 * This view is part of the focus example perspective.
 * It shows a JTable and a canvas.
 */
public class FocusExampleView extends ViewPart {

    private Object[][] rowData;
    private Object[] columnNames;

    public FocusExampleView(Object[][] tableRowData, Object[] tableColumnNames) {
        rowData = tableRowData;
        columnNames = tableColumnNames;
    }

    private SwingControl swingControl;

    static class MyCanvas extends JPanel {
        Point center;
        public void paintComponent(Graphics g) {
            if (center != null) {
                Rectangle rect = getBounds();
                double maxDist = 0;
                {
                    int x = rect.x - center.x;
                    int y = rect.y - center.y;
                    double dist = Math.sqrt(x*x+y*y);
                    if (maxDist < dist)
                        maxDist = dist;
                }
                {
                    int x = (rect.x+rect.width) - center.x;
                    int y = rect.y - center.y;
                    double dist = Math.sqrt(x*x+y*y);
                    if (maxDist < dist)
                        maxDist = dist;
                }
                {
                    int x = rect.x - center.x;
                    int y = (rect.y+rect.height) - center.y;
                    double dist = Math.sqrt(x*x+y*y);
                    if (maxDist < dist)
                        maxDist = dist;
                }
                {
                    int x = (rect.x+rect.width) - center.x;
                    int y = (rect.y+rect.height) - center.y;
                    double dist = Math.sqrt(x*x+y*y);
                    if (maxDist < dist)
                        maxDist = dist;
                }
                g.setColor(Color.white);
                g.fillRect(rect.x, rect.y, rect.width, rect.height);
                g.setColor(Color.black);
                for (int i = 0; 5*i <= maxDist; i++) {
                    int radius = 5*i;
                    g.drawArc(center.x-radius, center.y-radius, 2*radius, 2*radius, 0, 360);
                }
            }
        }
    }

    public void createPartControl(final Composite parent) {
        swingControl = new SwingControl(parent, SWT.NONE) {
            protected JComponent createSwingComponent() {
                /* Creating components */

                JTable table = new JTable(rowData, columnNames);

                JScrollPane scrollTable = new JScrollPane(table);

                final MyCanvas canvas = new MyCanvas();
                canvas.setBackground(Color.gray);
                canvas.setOpaque(true);
                canvas.addMouseListener(
                    new MouseAdapter() {
                        public void mouseClicked(MouseEvent event) {
                            // Yes, some predefined Components call requestFocus()
                            // when the user clicks in them.
                            canvas.requestFocus();

                            canvas.center = new Point(event.getX(), event.getY());
                            canvas.repaint();
                        }
                    });

                JPanel panel = new JPanel();
                panel.setLayout(new GridLayout(1, 2));
                panel.add(scrollTable);
                panel.add(canvas);

                return panel;
            }

            public Composite getLayoutAncestor() {
                return parent;
            }
            
        };
    }

    public void setFocus() {
        swingControl.setFocus();
    }

}
