package org.eclipse.albireo.examples.plugin.views;

import java.awt.BorderLayout;

import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.eclipse.albireo.core.SwingControl;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

public class FocusTraversalView extends ViewPart {
    private static final int SWT_ONLY = 0;
    private static final int SWING_ONLY = 1;
    private static final int MIXED = 2;
    
    public static String ID="org.eclipse.albireo.examples.plugin.focusTraversalView"; //$NON-NLS-1$
    private Composite composite;
    private int mode = SWING_ONLY;
    private Composite fields;

    public void createPartControl(Composite parent) {
        composite = new Composite(parent, SWT.NONE);
        composite.setLayout(new GridLayout());
        
        createGroup();
        
        Control separator = new Label(composite, SWT.SEPARATOR | SWT.HORIZONTAL);
        GridData data = new GridData();
        data.horizontalAlignment = SWT.FILL;
        data.verticalAlignment = SWT.CENTER;
        data.grabExcessHorizontalSpace = true;
        separator.setLayoutData(data);
        
        fields = createFields(composite);
        data = new GridData();
        data.horizontalAlignment = SWT.FILL;
        data.verticalAlignment = SWT.CENTER;
        data.grabExcessHorizontalSpace = true;
        data.grabExcessVerticalSpace = true;
        fields.setLayoutData(data);
    }

    private void createGroup() {
        Group group = new Group(composite, SWT.NONE);
        GridLayout layout = new GridLayout();
        layout.numColumns = 1;
        group.setLayout(layout);
        GridData data = new GridData();
        data.horizontalAlignment = SWT.FILL;
        data.verticalAlignment = SWT.FILL;
        group.setLayoutData(data);
        
        
        Button[] radios = new Button[3];

        radios[0] = new Button(group, SWT.RADIO);
        radios[0].setText("Swing");
        radios[0].addSelectionListener(new SelectionListener() {
            public void widgetDefaultSelected(SelectionEvent e) {
            }
            public void widgetSelected(SelectionEvent e) {
                if (mode != SWING_ONLY) {
                    mode = SWING_ONLY;
                    updateFields();
                }
            }
        });
        data = new GridData();
        data.horizontalAlignment = SWT.LEFT;
        data.verticalAlignment = SWT.CENTER;
        radios[0].setLayoutData(data);
        

        radios[1] = new Button(group, SWT.RADIO);
        radios[1].setText("SWT");
        radios[1].addSelectionListener(new SelectionListener() {
            public void widgetDefaultSelected(SelectionEvent e) {
            }
            public void widgetSelected(SelectionEvent e) {
                if (mode != SWT_ONLY) {
                    mode = SWT_ONLY;
                    updateFields();
                }
            }
        });
        data = new GridData();
        data.horizontalAlignment = SWT.LEFT;
        data.verticalAlignment = SWT.CENTER;
        radios[1].setLayoutData(data);
        

        radios[2] = new Button(group, SWT.RADIO);
        radios[2].setText("Mixed");
        radios[2].addSelectionListener(new SelectionListener() {
            public void widgetDefaultSelected(SelectionEvent e) {
            }
            public void widgetSelected(SelectionEvent e) {
                if (mode != MIXED) {
                    mode = MIXED;
                    updateFields();
                }
            }
        });
        data = new GridData();
        data.horizontalAlignment = SWT.LEFT;
        data.verticalAlignment = SWT.CENTER;
        radios[2].setLayoutData(data);
        
        radios[0].setSelection(true);
    }

    protected void updateFields() {
        fields.dispose();
        fields = createFields(composite);
        fields.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        composite.layout();
    }

    private Composite createFields(Composite composite) {
        switch (mode) {
        case SWING_ONLY:
            return createSwingOnly(composite);

        case SWT_ONLY:
            return createSWTOnly(composite);

        case MIXED:
            return createMixed(composite);
        }
        return null;
    }

    private Composite createMixed(Composite composite) {
        Composite c = new Composite(composite, SWT.NONE);
        c.setLayout(new GridLayout());
        Control control = createSwingField(c);
        control.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, true));
        control = createSWTField(c);
        control.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, true));
        return c;
    }

    private Composite createSWTOnly(Composite composite) {
        Composite c = new Composite(composite, SWT.NONE);
        c.setLayout(new GridLayout());
        Control control = createSWTField(c);
        control.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, true));
        control = createSWTField(c);
        control.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, true));
        return c;
    }

    private Composite createSwingOnly(Composite composite) {
        Composite c = new Composite(composite, SWT.NONE);
        c.setLayout(new GridLayout());
        Control control = createSwingField(c);
        control.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, true));
        control = createSwingField(c);
        control.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, true));
        return c;
    }

    public void setFocus() {
        composite.setFocus();
    }

    private Control createSwingField(final Composite parent) {
        Control control = new SwingControl(parent, SWT.NONE) {

            protected JComponent createSwingComponent() {
                JLabel label = new JLabel("Swing Field: ");
                JTextField text = new JTextField("some Swing text");
                
                JPanel panel = new JPanel(new BorderLayout());
                panel.add(label, BorderLayout.LINE_START);
                panel.add(text, BorderLayout.LINE_END);

                return panel;
            }

            public Composite getLayoutAncestor() {
                return getParent();
            }
            
        };
        return control;
    }

    private Composite createSWTField(Composite parent) {
        Composite composite = new Composite(parent, SWT.NONE);
        composite.setLayout(new GridLayout(2, false));
        Label label = new Label(composite, SWT.NONE);
        label.setText("SWT Field: ");
        label.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false));
        Text text = new Text(composite, SWT.NONE);
        text.setText("some SWT text");
        text.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false));
        return composite;
    }


}
