package org.eclipse.albireo.examples.plugin.views;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Frame;
import java.awt.Panel;

import javax.swing.JButton;
import javax.swing.JComponent;

import org.eclipse.albireo.core.SwingControl;
import org.eclipse.swt.SWT;
import org.eclipse.swt.awt.SWT_AWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.part.ViewPart;

/**
 * This view allows to check against the SWT bug #58308.
 * https://bugs.eclipse.org/bugs/show_bug.cgi?id=58308
 */
public class SWTBug58308 extends ViewPart {

    public static String ID = "org.eclipse.albireo.examples.plugin.58308"; //$NON-NLS-1$

    Control swingControl;

    public void createPartControl(final Composite parent) {
        final JButton button = new JButton();
        button.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
        if (false) {
            // ORIGINAL CODE
            // Broken: cross-hair cursor in wrong area
            Composite comp = new Composite(parent, SWT.EMBEDDED);                
            Frame frame = SWT_AWT.new_Frame(comp);
            frame.add(button);
            swingControl = comp;
        } else if (false) {
            // ORIGINAL CODE, as corrected in comment #10 of the above bug
            // Works.
            Composite comp = new Composite(parent, SWT.EMBEDDED);                
            Frame frame = SWT_AWT.new_Frame(comp);
            Panel panel = new Panel(new BorderLayout());
            frame.add(panel);
            panel.add(button);
            swingControl = comp;
        } else {
            // ALBIREO CODE
            // Works.
            swingControl = 
                new SwingControl(parent, SWT.NONE) {
                    protected JComponent createSwingComponent() {
                        return button;
                    }
                    public Composite getLayoutAncestor() {
                        return parent;
                    }
                };
        }
    }

    public void setFocus() {
        swingControl.setFocus();
    }

}
