package org.eclipse.albireo.examples.plugin.views;

import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import org.eclipse.albireo.core.SwingControl;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

/**
 * Used by the {@link RelayoutExampleView}.
 */
abstract class SwingGrowShrinkControl extends SwingControl {
    private static final String PREFIX = "Count: ";
    private int countInt;
    private JLabel count;
    private List labels = new ArrayList();
    private JButton growButton;
    private JButton shrinkButton;
    private JButton layoutButton;
    private JPanel panel;

    SwingGrowShrinkControl(Composite parent, int style) {
        super(parent, style);
    }

    protected JComponent createSwingComponent() {
        addSwingWidgets();
        return panel;
    }

    private void addSwingWidgets() {
        panel = new JPanel(new java.awt.GridLayout(0, 1));

        countInt = 1;
        count = new JLabel(PREFIX + String.valueOf(countInt), SwingConstants.LEFT); 

        JLabel label = new JLabel("Abc ", SwingConstants.LEFT);
        labels.add(label);

        growButton = new JButton("Grow");
        shrinkButton = new JButton("Shrink");
        shrinkButton.setEnabled(false);
        layoutButton = new JButton("SWT-Layout");

        panel.add(count);
        panel.add(label);
        panel.add(growButton);
        panel.add(shrinkButton);
        panel.add(layoutButton);

        growButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    growOrShrink(true);
                }
            });
        shrinkButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    growOrShrink(false);
                }
            });
        layoutButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    getDisplay().asyncExec(
                        new Runnable() {
                            public void run() {
                                relayoutFromOutside();
                            }
                        });
                }
            });
    }

    protected void growOrShrink(boolean grow) {
        assert EventQueue.isDispatchThread();    // On AWT event thread

        String text = ((JLabel)labels.get(0)).getText();
        if (grow) {
            text += "Abc ";
            countInt++;
            shrinkButton.setEnabled(true);
        } else {
            if (!isShrinkButtonEnabled())
                return;
            text = text.substring(4);
            countInt--;
            if (text.length() <= 4) {
                shrinkButton.setEnabled(false);
            }
        }
        count.setText(PREFIX + String.valueOf(countInt));
        if (grow) {
            JLabel label = new JLabel("Abc ", SwingConstants.LEFT);
            labels.add(label);
            panel.add(label, 1);
        } else {
            JLabel label = (JLabel)labels.remove(0);
            panel.remove(label);
        }
        for (Iterator iterator = labels.iterator(); iterator.hasNext();) {
            JLabel label = (JLabel)iterator.next();
            label.setText(text);
        }
    }

    public boolean isShrinkButtonEnabled() {
        return shrinkButton.isEnabled();
    }

    void relayoutFromOutside() {
        assert Display.getCurrent() != null;     // On SWT event thread

        getLayoutAncestor().layout();
    }

}
