package org.eclipse.albireo.examples.plugin.views;

import java.awt.Frame;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;

import javax.swing.JApplet;
import javax.swing.JComponent;
import javax.swing.RootPaneContainer;

import org.eclipse.albireo.core.SwingControl;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

/**
 * This example allows for testing and experimentation with a wide variety of
 * Swing controls by embedding the standard Sun SwingSet applet in an RCP view.
 * This does not work perfectly, and, at least for now, the Albireo API does
 * not smoothly support the integration of an applet like this.
 *
 * Note! This view only works if it was closed when Eclipse was launched
 * and it is the first time you open it. It does *not* work (it displays only
 * a splash screen) if it opens when Eclipse starts (restoring the previous
 * session's views), or the second time you open it in the same Eclipse
 * session. This must be due to the internals of the SwingSet2 class.
 */
public class SwingSetView extends ViewPart {

    private SwingControl control;
    private static File jarFile;
    private JApplet applet;
    private ScrolledComposite scrolledComposite;

    public SwingSetView() {
    }

    public void createPartControl(Composite parent) {
        findSwingSet();
        if (jarFile == null) {
            return;
        }
        scrolledComposite = new ScrolledComposite(parent, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
        control = new SwingControl(scrolledComposite, SWT.NONE) {

            protected RootPaneContainer addRootPaneContainer(Frame frame) {
                // TODO: this code does not add the appropriate updates to the AWT sizes cache. 
                try {
                    // This is a hack to avoid the need to include SwingSet source/binaries in this 
                    // example project by loading it up through a separate classloader and using 
                    // reflection. As with much of this example, it is not recommended behavior for 
                    // a real app. 
                    URL[] urls = new URL[1];
                    urls[0] = jarFile.toURL();
                    URLClassLoader classLoader = new URLClassLoader(urls);
                    Class clazz = classLoader.loadClass("SwingSet2Applet");
                    applet = (JApplet)clazz.newInstance();
                } catch (MalformedURLException e) {
                    throw new RuntimeException(e);
                } catch (ClassNotFoundException e) {
                    throw new RuntimeException(e);
                } catch (InstantiationException e) {
                    throw new RuntimeException(e);
                } catch (IllegalAccessException e) {
                    throw new RuntimeException(e);
                }
                frame.add(applet);
                
                // Emulate a browser by starting up the applet
                applet.init();
                applet.start();
                
                return applet;
            }

            protected JComponent createSwingComponent() {
                return null;
            }

            public Composite getLayoutAncestor() {
                // Size is fixed within scrolled composite
                return null;
            }
        };
        
        // TODO: the fixed size is because SwingSet does not automatically resize. Investigate. 
        control.setSize(850, 650);
        scrolledComposite.setContent(control);
    }

    private void findSwingSet() {
        // Try to find SwingSet2.jar if the application was run from a JDK that has the demo code
        // installed.
        if (jarFile == null) {
            File file = new File(System.getProperty("java.home") + "/../demo/jfc/SwingSet2/SwingSet2.jar");
            while (!file.exists()) {
                // Prompt user for jar location if it was not found
                // TODO: try to access SwingSet from sun.com?
                //        (e.g. http://java.sun.com/products/plugin/1.5.0/demos/jfc/SwingSet2/SwingSet2.jar)
                // TODO: A more descriptive dialog would be nice
                FileDialog dlg = new FileDialog(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell());
                dlg.setText("Select a location for SwingSet2.jar");
                String name = dlg.open();
                if (name == null) {
                    return;
                }
                file = new File(name);
            }
            jarFile = file;
        }
    }

    public void setFocus() {
        control.setFocus();
    }

}
