package org.eclipse.albireo.examples.plugin.views;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.Window;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.border.Border;
import javax.swing.border.MatteBorder;

import org.eclipse.albireo.core.SwingControl;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.part.ViewPart;

/**
 * This view allows to check the conversion between component coordinates,
 * window coordinates, and screen coordinates.
 */
public class TestScreenCoordinatesView extends ViewPart {

    public static String ID = "org.eclipse.albireo.examples.plugin.testScreenCoordinates"; //$NON-NLS-1$
    private SwingControl swingControl;

    public void createPartControl(final Composite parent) {
        swingControl = new SwingControl(parent, SWT.NONE) {
            protected JComponent createSwingComponent() {
                JPanel entirePanel = new JPanel();
                entirePanel.setLayout(new BorderLayout());

                JPanel bottomPanel = new JPanel();
                final JLabel innerCoordinates;
                final JLabel quarterCoordinates;
                final JLabel awtWindowCoordinates;
                final JLabel screenCoordinates;
                bottomPanel.setLayout(new BoxLayout(bottomPanel, BoxLayout.PAGE_AXIS));
                {
                    Box line = new Box(BoxLayout.LINE_AXIS);
                    JLabel label = new JLabel("Coordinates relative to green rectangle: ");
                    label.setFont(new Font("Dialog", Font.BOLD, 12));
                    line.add(label);
                    JLabel coordinates = new JLabel("");
                    coordinates.setMinimumSize(new Dimension(100, 20));
                    coordinates.setPreferredSize(coordinates.getMinimumSize());
                    line.add(coordinates);
                    line.add(Box.createHorizontalGlue());
                    bottomPanel.add(line);
                    innerCoordinates = coordinates;
                }
                {
                    Box line = new Box(BoxLayout.LINE_AXIS);
                    JLabel label = new JLabel("Coordinates relative to orange rectangle: ");
                    label.setFont(new Font("Dialog", Font.BOLD, 12));
                    line.add(label);
                    JLabel coordinates = new JLabel("");
                    coordinates.setMinimumSize(new Dimension(100, 20));
                    coordinates.setPreferredSize(coordinates.getMinimumSize());
                    line.add(coordinates);
                    line.add(Box.createHorizontalGlue());
                    bottomPanel.add(line);
                    quarterCoordinates = coordinates;
                }
                {
                    Box line = new Box(BoxLayout.LINE_AXIS);
                    JLabel label = new JLabel("Coordinates relative to AWT window: ");
                    label.setFont(new Font("Dialog", Font.BOLD, 12));
                    line.add(label);
                    JLabel coordinates = new JLabel("");
                    coordinates.setMinimumSize(new Dimension(100, 20));
                    coordinates.setPreferredSize(coordinates.getMinimumSize());
                    line.add(coordinates);
                    line.add(Box.createHorizontalGlue());
                    bottomPanel.add(line);
                    awtWindowCoordinates = coordinates;
                }
                {
                    Box line = new Box(BoxLayout.LINE_AXIS);
                    JLabel label = new JLabel("Coordinates relative to screen: ");
                    label.setFont(new Font("Dialog", Font.BOLD, 12));
                    line.add(label);
                    JLabel coordinates = new JLabel("");
                    coordinates.setMinimumSize(new Dimension(100, 20));
                    coordinates.setPreferredSize(coordinates.getMinimumSize());
                    line.add(coordinates);
                    line.add(Box.createHorizontalGlue());
                    bottomPanel.add(line);
                    screenCoordinates = coordinates;
                }
                entirePanel.add(bottomPanel, BorderLayout.SOUTH);

                class InnerMouseListener implements MouseListener, MouseMotionListener {
                    private void react(MouseEvent event) {
                        int x = event.getX();
                        int y = event.getY();
                        Component inner = event.getComponent();
                        innerCoordinates.setText("("+x+","+y+")");
                        java.awt.Point p3 = SwingUtilities.convertPoint(inner, x, y,
                                                                        inner.getParent());
                        quarterCoordinates.setText("("+p3.x+","+p3.y+")");
                        Window w = SwingUtilities.getWindowAncestor(inner);
                        if (w != null) {
                            java.awt.Point p2 = SwingUtilities.convertPoint(inner, x, y, w);
                            awtWindowCoordinates.setText("("+p2.x+","+p2.y+")");
                        } else {
                            awtWindowCoordinates.setText("");
                        }
                        java.awt.Point p1 = new java.awt.Point(x, y);
                        SwingUtilities.convertPointToScreen(p1, inner);
                        screenCoordinates.setText("("+p1.x+","+p1.y+")");
                    }
                    public void mouseClicked(MouseEvent event) {
                        react(event);
                    }
                    public void mouseEntered(MouseEvent event) {
                        react(event);
                    }
                    public void mouseExited(MouseEvent event) {
                        react(event);
                    }
                    public void mousePressed(MouseEvent event) {
                        react(event);
                    }
                    public void mouseReleased(MouseEvent event) {
                        react(event);
                    }
                    public void mouseDragged(MouseEvent event) {
                        react(event);
                    }
                    public void mouseMoved(MouseEvent event) {
                        react(event);
                    }
                }

                class QuarterMouseListener implements MouseListener, MouseMotionListener {
                    private void react(MouseEvent event) {
                        int x = event.getX();
                        int y = event.getY();
                        Component quarter = event.getComponent();
                        innerCoordinates.setText("");
                        quarterCoordinates.setText("("+x+","+y+")");
                        Window w = SwingUtilities.getWindowAncestor(quarter);
                        if (w != null) {
                            java.awt.Point p2 = SwingUtilities.convertPoint(quarter, x, y, w);
                            awtWindowCoordinates.setText("("+p2.x+","+p2.y+")");
                        } else {
                            awtWindowCoordinates.setText("");
                        }
                        java.awt.Point p1 = new java.awt.Point(x, y);
                        SwingUtilities.convertPointToScreen(p1, quarter);
                        screenCoordinates.setText("("+p1.x+","+p1.y+")");
                    }
                    public void mouseClicked(MouseEvent event) {
                        react(event);
                    }
                    public void mouseEntered(MouseEvent event) {
                        react(event);
                    }
                    public void mouseExited(MouseEvent event) {
                        react(event);
                    }
                    public void mousePressed(MouseEvent event) {
                        react(event);
                    }
                    public void mouseReleased(MouseEvent event) {
                        react(event);
                    }
                    public void mouseDragged(MouseEvent event) {
                        react(event);
                    }
                    public void mouseMoved(MouseEvent event) {
                        react(event);
                    }
                }

                InnerMouseListener innerMouseListener =
                    new InnerMouseListener();

                QuarterMouseListener quarterMouseListener =
                    new QuarterMouseListener();

                JPanel panel = new JPanel();
                panel.setLayout(new GridLayout(2, 2));

                Border border = new MatteBorder(5, 5, 5, 5, Color.orange);
                for (int i = 0; i < 4; i++) {

                    JPanel quarterPanel = new JPanel();
                    quarterPanel.setBorder(border);
                    quarterPanel.setLayout(new BorderLayout());

                    JPanel innerPanel = new JPanel();
                    innerPanel.setOpaque(true);
                    innerPanel.setBackground(Color.green);
                    innerPanel.addMouseListener(innerMouseListener);
                    innerPanel.addMouseMotionListener(innerMouseListener);
                    
                    quarterPanel.add(innerPanel, BorderLayout.CENTER);
                    quarterPanel.addMouseListener(quarterMouseListener);
                    quarterPanel.addMouseMotionListener(quarterMouseListener);
                    panel.add(quarterPanel);
                }
                entirePanel.add(panel, BorderLayout.CENTER);

                return entirePanel;
            }

            public Composite getLayoutAncestor() {
                return parent;
            }
        };
    }

    public void setFocus() {
        swingControl.setFocus();
    }
}
