package org.eclipse.albireo.examples.plugin.views;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableCellRenderer;
import javax.swing.text.JTextComponent;

import org.eclipse.albireo.core.SwingControl;
import org.eclipse.core.runtime.Platform;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.internal.util.BundleUtility;
import org.eclipse.ui.part.ViewPart;
import org.osgi.framework.Bundle;

/**
 * This view shows an exhibition of various Swing components.
 */
public class ZooView extends ViewPart {

    public static String ID = "org.eclipse.albireo.examples.plugin.zoo"; //$NON-NLS-1$
    private SwingControl swingControl;

    private static URL getResource(String relativeFilename) {
        // Code borrowed from
        // org.eclipse.ui.plugin.AbstractUIPlugin.imageDescriptorFromPlugin.
        Bundle bundle = Platform.getBundle("org.eclipse.albireo.examples.plugin");
        if (BundleUtility.isReady(bundle)) {
            URL url = BundleUtility.find(bundle, relativeFilename);
            if (url != null)
                return url;
            try {
                return new URL(relativeFilename);
            } catch (MalformedURLException e) {
                e.printStackTrace();
            }
        }
        return null;
    }

    public void createPartControl(final Composite parent) {
        swingControl = new SwingControl(parent, SWT.NONE) {
            protected JComponent createSwingComponent() {
                JScrollPane scrollable;
                {
                    JPanel panel = new JPanel();
                    panel.setLayout(new FlowLayout());

                    addButtons(panel);
                    addComboBox(panel);
                    addList(panel);
                    addTexts(panel);
                    addTable(panel);

                    scrollable = new JScrollPane(panel);
                }
                return scrollable;
            }
            public Composite getLayoutAncestor() {
                return parent;
            }
        };
    }

    private void addButtons(JComponent panel) {

        JButton button1 =
            new JButton("Click here",
                        new ImageIcon(
                            Toolkit.getDefaultToolkit().createImage(getResource("icons/alt_window_32.gif"))));
        button1.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    System.out.println("button1 clicked");
                }
            });
        button1.addChangeListener(
            new ChangeListener() {
                public void stateChanged(ChangeEvent event) {
                    System.out.println("button1 sent "+event);
                }
            });
        panel.add(button1);

        final JToggleButton button2 =
            new JToggleButton("Press",
                              new ImageIcon(
                                  Toolkit.getDefaultToolkit().createImage(getResource("icons/alt_window_32.gif"))));
        button2.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    if (button2.isSelected()) {
                        System.out.println("button2 now on");
                        button2.setText("Press again");
                    } else {
                        System.out.println("button2 now off");
                        button2.setText("Press");
                    }
                }
            });
        button2.addChangeListener(
            new ChangeListener() {
                public void stateChanged(ChangeEvent event) {
                    System.out.println("button2 sent "+event);
                }
            });
        panel.add(button2);

        final JCheckBox checkbox1 =
            new JCheckBox("Encrypt");
        checkbox1.addActionListener(
              new ActionListener() {
                  public void actionPerformed(ActionEvent event) {
                      if (checkbox1.isSelected()) {
                          System.out.println("checkbox1 now on");
                      } else {
                          System.out.println("checkbox1 now off");
                      }
                  }
              });
        checkbox1.addChangeListener(
              new ChangeListener() {
                  public void stateChanged(ChangeEvent event) {
                       System.out.println("checkbox1 sent "+event);
                  }
              });
        panel.add(checkbox1);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.PAGE_AXIS));
        {
            ButtonGroup rbuttons = new ButtonGroup();

            final JRadioButton rbutton1 =
                new JRadioButton("Clinton");
            rbutton1.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent event) {
                        if (rbutton1.isSelected()) {
                            System.out.println("rbutton1 now on");
                        } else {
                            System.out.println("rbutton1 now off");
                        }
                    }
                });
            rbutton1.addChangeListener(
                new ChangeListener() {
                    public void stateChanged(ChangeEvent event) {
                        System.out.println("rbutton1 sent "+event);
                    }
                });
            rbuttons.add(rbutton1);
            buttonPanel.add(rbutton1);

            final JRadioButton rbutton2 =
                new JRadioButton("Obama");
            rbutton2.addActionListener(
                new ActionListener() {
                    public void actionPerformed(ActionEvent event) {
                        if (rbutton2.isSelected()) {
                            System.out.println("rbutton2 now on");
                        } else {
                            System.out.println("rbutton2 now off");
                        }
                    }
                });
            rbutton2.addChangeListener(
                new ChangeListener() {
                    public void stateChanged(ChangeEvent event) {
                        System.out.println("rbutton2 sent "+event);
                    }
                });
            rbuttons.add(rbutton2);
            buttonPanel.add(rbutton2);
        }
        panel.add(buttonPanel);

    }

    // Many choices, to test what happens when the combobox
    // does not fit into the AWT part.
    static final String[] comboboxChoices =
        new String[] {
            "Afar",
            "Abkhazian",
            "Adangme",
            "Avestan",
            "Afrikaans",
            "Akan",
            "Amharic",
            "Aragonese",
            "Arabic",
            "Assamese",
            "Avaric",
            "Aymara",
            "Azerbaijani",
            "Bashkir",
            "Belarusian",
            "Bulgarian",
            "Bihari",
            "Bislama",
            "Bambara",
            "Bengali",
            "Tibetan",
            "Breton",
            "Bosnian",
            "Catalan",
            "Chechen",
            "Chamorro",
            "Corsican",
            "Cree",
            "Czech",
            "Church Slavic",
            "Chuvash",
            "Welsh",
            "Danish",
            "German",
            "Divehi",
            "Dzongkha",
            "\u00c9w\u00e9",
            "Greek",
            "English",
            "Esperanto",
            "Spanish",
            "Estonian",
            "Basque",
            "Persian",
            "Fulah",
            "Finnish",
            "Fijian",
            "Faroese",
            "French",
            "Western Frisian",
            "Irish",
            "Gaelic",
            "Galician",
            "Guarani",
            "Gujarati",
            "Manx",
            "Hausa",
            "Hebrew",
            "Hindi",
            "Hiri Motu",
            "Croatian",
            "Haitian",
            "Hungarian",
            "Armenian",
            "Herero",
            "Interlingua",
            "Indonesian",
            "Interlingue",
            "Igbo",
            "Sichuan Yi",
            "Inupiak",
            "Ido",
            "Icelandic",
            "Italian",
            "Inuktitut",
            "Japanese",
            "Javanese",
            "Georgian",
            "Kongo",
            "Kikuyu",
            "Kuanyama",
            "Kazakh",
            "Kalaallisut",
            "Khmer",
            "Kannada",
            "Korean",
            "Kanuri",
            "Kashmiri",
            "Kurdish",
            "Komi",
            "Cornish",
            "Kirghiz",
            "Latin",
            "Luxembourgish",
            "Ganda",
            "Limburgish",
            "Lingala",
            "Laotian",
            "Lithuanian",
            "Luba-Katanga",
            "Latvian",
            "Malagasy",
            "Marshallese",
            "Maori",
            "Macedonian",
            "Malayalam",
            "Mongolian",
            "Moldavian",
            "Marathi",
            "Malay",
            "Maltese",
            "Burmese",
            "Nauru",
            "Norwegian Bokm\u00e5l",
            "Ndebele, North",
            "Nepali",
            "Ndonga",
            "Dutch",
            "Norwegian Nynorsk",
            "Norwegian",
            "Ndebele, South",
            "Navaho",
            "Chichewa",
            "Occitan",
            "Ojibwa",
            "(Afan) Oromo",
            "Oriya",
            "Ossetian",
            "Punjabi",
            "Pali",
            "Polish",
            "Pashto",
            "Portuguese",
            "Quechua",
            "Rhaeto-Romance",
            "Kirundi",
            "Romanian",
            "Russian",
            "Kinyarwanda",
            "Sanskrit",
            "Sardinian",
            "Sindhi",
            "Northern Sami",
            "Sango",
            "Sinhala",
            "Slovak",
            "Slovenian",
            "Samoan",
            "Shona",
            "Somali",
            "Albanian",
            "Serbian",
            "Siswati",
            "Sesotho",
            "Sundanese",
            "Swedish",
            "Swahili",
            "Tamil",
            "Telugu",
            "Tajik",
            "Thai",
            "Tigrinya",
            "Turkmen",
            "Tagalog",
            "Setswana",
            "Tonga",
            "Turkish",
            "Tsonga",
            "Tatar",
            "Twi",
            "Tahitian",
            "Uighur",
            "Ukrainian",
            "Urdu",
            "Uzbek",
            "Venda",
            "Vietnamese",
            "Volap\u00fck",
            "Walloon",
            "Wolof",
            "Xhosa",
            "Yiddish",
            "Yoruba",
            "Zhuang",
            "Chinese",
            "Zulu",
        };

    private void addComboBox(JComponent panel) {

        final JComboBox combobox1 =
            new JComboBox(comboboxChoices);
        combobox1.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    System.out.println("combobox1 now selected: "+combobox1.getSelectedItem());
                }
            });
        combobox1.addItemListener(
            new ItemListener() {
                public void itemStateChanged(ItemEvent event) {
                    System.out.println("combobox1 sent "+event);
                }
            });
        panel.add(combobox1);

    }

    private void addList(JComponent panel) {

        JList list1 = new JList(comboboxChoices);
        list1.addListSelectionListener(
            new ListSelectionListener() {
                public void valueChanged(ListSelectionEvent event) {
                    System.out.println("list1 send "+event);
                }
            });
        panel.add(list1);

    }

    private void addTexts(JComponent panel) {
        JPanel textsPanel = new JPanel();
        textsPanel.setLayout(new BoxLayout(textsPanel, BoxLayout.PAGE_AXIS));

        JTextField textField1 =
            new JTextField("initial text");
        textField1.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    System.out.println("textField1 sent "+event);
                }
            });
        textField1.getDocument().addDocumentListener(
            new DocumentListener() {
                public void changedUpdate(DocumentEvent event) {
                    System.out.println("textField1 changed "+event);
                }
                public void insertUpdate(DocumentEvent event) {
                    System.out.println("textField1 insert "+event);
                }
                public void removeUpdate(DocumentEvent event) {
                    System.out.println("textField1 remove "+event);
                }
            });
        textsPanel.add(textField1);

        JTextField textField2 =
            new JTextField("initial text");
        textField2.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    System.out.println("textField2 sent "+event);
                }
            });
        textField2.getDocument().addDocumentListener(
            new DocumentListener() {
                public void changedUpdate(DocumentEvent event) {
                    System.out.println("textField2 changed "+event);
                }
                public void insertUpdate(DocumentEvent event) {
                    System.out.println("textField2 insert "+event);
                }
                public void removeUpdate(DocumentEvent event) {
                    System.out.println("textField2 remove "+event);
                }
            });
        textsPanel.add(textField2);

        String textArea1String =
            "Many adopters of the Eclipse Rich Client Platform (RCP) start with large component libraries that target the Swing user interface toolkit. These libraries provide high business value in specific application domains and cannot be replaced with a third-party SWT library. Since a complete, immediate conversion to SWT may be impractical, these developers need a way to embed the existing Swing components into their new RCP applications as they begin the transition to the RCP.\n" 
            +"Other developers, who are creating new RCP or SWT applications, may be forced to use third-party Swing component libraries simply because there are no existing SWT equivalents.\n" 
            +"Since Eclipse 3.0, SWT has addressed these use cases with the basic Swing/SWT integration support in the SWT_AWT bridge. The bridge provides the critical infrastructure necessary to integrate the two toolkits, but it takes a very minimal approach. The recent Eclipse Corner article on Swing/SWT Integration describes several additional techniques required to effectively combine Swing and SWT components into production-quality applications. So, while the SWT_AWT bridge makes integration possible, it is still quite difficult.\n" 
            +"By providing a higher-level Swing/SWT integration framework, the Albireo project will make the reuse of existing Swing components easier and further reduce the \"barriers to entry\" encountered by new RCP adopters.\n";
        JTextArea textArea1 =
            new JTextArea(textArea1String);
        textArea1.setLineWrap(true);
        textArea1.setPreferredSize(new Dimension(400, 500));
        textArea1.getDocument().addDocumentListener(
            new DocumentListener() {
                public void changedUpdate(DocumentEvent event) {
                    System.out.println("textArea1 changed "+event);
                }
                public void insertUpdate(DocumentEvent event) {
                    System.out.println("textArea1 insert "+event);
                }
                public void removeUpdate(DocumentEvent event) {
                    System.out.println("textArea1 remove "+event);
                }
            });
        textsPanel.add(textArea1);

        panel.add(textsPanel);
    }

    private void addTable(JComponent panel) {

        String[] table1Columns =
            new String[] { "column1", "column2" };
        String[][] table1Contents =
            new String[][] {
                new String[] {
                    "Ebony",
                    "Ivory"
                },
                new String[] {
                    "<<Dark wood>>",
                    "<<Expensive>>"
                },
                new String[] {
                    "Dark wood of a large tree",
                    "Expensive elefant product"
                }
            };
        JTable table1 =
            new JTable(table1Contents, table1Columns) {
                // For testing tooltips.
                public String getToolTipText(MouseEvent event) {
                    // On Windows, don't show tooltips when
                    // this component is not in the focused window.
                    if (java.io.File.separatorChar == '\\') {
                        Container toplevel = getTopLevelAncestor();
                        if (toplevel instanceof Window) {
                            if (!((Window) toplevel).isActive())
                                return null;
                        }
                    }
                    // Use the tooltip from the renderer if
                    // available.
                    String tooltipFromRenderer = super.getToolTipText(event);
                    if (tooltipFromRenderer != null)
                        return tooltipFromRenderer;
                    // Determine the cell over which the mouse is.
                    Point p = event.getPoint();
                    int hitRowIndex = rowAtPoint(p);
                    int hitColumnIndex = columnAtPoint(p);
                    if (hitRowIndex != -1 && hitColumnIndex != -1) {
                        TableCellRenderer renderer = getCellRenderer(hitRowIndex,hitColumnIndex);
                        Component component = prepareRenderer(renderer,hitRowIndex,hitColumnIndex);
                        if (component instanceof JLabel) {
                            return ((JLabel)component).getText();
                        }
                        if (component instanceof JTextComponent) {
                            return ((JTextComponent)component).getText();
                        }
                        int modelColumnIndex = convertColumnIndexToModel(hitColumnIndex);
                        Object value = getModel().getValueAt(hitRowIndex,modelColumnIndex);
                        if (value != null && !(value instanceof JComponent)) {
                            return value.toString();
                        }
                    }
                    return null;
                }
            };
        table1.setMaximumSize(new Dimension (100, 70));
        table1.setPreferredSize(table1.getMaximumSize());
        panel.add(table1);

    }

    public void setFocus() {
        swingControl.setFocus();
    }
}
